with SYSTEM;              use SYSTEM;
with CSSW_COMMON_TYPES;   use CSSW_COMMON_TYPES;
with CSSW_IF_TYPES;       use CSSW_IF_TYPES;
with IASW_COMMON;

package IASW_DFEE is

   package CIT renames CSSW_IF_TYPES;
   package CCT renames CSSW_COMMON_TYPES;

   -- Exceptions raised in this package
   EX_DFEE_CALLER_ERROR,   -- You have called a procedure with wrong params.
   EX_DFEE_DETECTS_ERROR,    -- DFEE protests executing the request
   EX_DFEE_UNEXP_ANSWER,     -- DFEE has answered, but not convinient way
   EX_LSL_DEAD,              -- No communication on LSL
   EX_DFEE_CRC_HAS_CHANGED,  -- Bad CRCs after recovering
   EX_DFEE_BUSY              -- DFEE is busy with exec. of previous op.
      : exception;
   -- moved here from IASW_PATCHES (which is linked in AS14)
   -- because of problems in transfering exceptions across A.S.
   EX_PATCHES_TABLE_FULL,    -- raised in ADD_PATCH()
   EX_PATCHES_LENGTH_ERROR,  -- raised in ADD_PATCH()
   EX_PATCHES_INDEX_ERROR    -- raised in GET_PATCH()
      : exception;

   -- Codes of the JEM-X state used in SET_STATE()
   subtype T_DFEE_STATE is CCT.T_UNSIGNED_16;

   K_STATE_STANDBY     : constant T_DFEE_STATE := 0;
   K_STATE_SAFE        : constant T_DFEE_STATE := 1;
   K_STATE_MEMORY      : constant T_DFEE_STATE := 2;
   K_STATE_SETUP       : constant T_DFEE_STATE := 5;
   K_STATE_SETUP_HVR   : constant T_DFEE_STATE := 6;
   K_STATE_DATA_TAKING : constant T_DFEE_STATE := 10;
   K_STATE_CALIBRATION : constant T_DFEE_STATE := 20;
   K_STATE_DIAGNOSTIC  : constant T_DFEE_STATE := 40;

   -- minimal HV values, being defaults after HV ON.
   MIN_ACC_HV   : constant CCT.T_UNSIGNED_16 := 16#12#;
   MIN_DRIFT_HV : constant CCT.T_UNSIGNED_16 := 16#12#;

   -- when automatically recovering HV, its values will be set a bit
   -- lower than nominal. the numbers below define this 'a bit'.
   -- they can be set wit TC(5,3),TID=0,FID=14
   DELTA_ACC_HV   : CCT.T_UNSIGNED_16 := 5;
   DELTA_DRIFT_HV : CCT.T_UNSIGNED_16 := 5;

   SW_PARAMS_INT_NO    : constant INTEGER            := 100;
   SW_PARAMS_FLOAT_NO  : constant INTEGER            := 100;

   K_MAX_LOAD_MEM_BLK_LEN : constant INTEGER := 200;
   -- 200 is the highest of the three numbers:
   -- 1)   CIT.T_TC_APPL_DATA_LEN'length - 7;  (=120-7) which is limit for TC load
   -- 2)   128, which is the length of loading of half of energy table
   -- 3)   SW_PARAMS_INT_NO (=100), which is the length of int params. table.
   -- 4)   SW_PARAMS_FLOAT_NO*2 (=100*2), which is the length of float params. table.
   K_MAX_DUMP_MEM_BLK_LEN : constant INTEGER := 
      CIT.T_SCIENCE_DATA_FIELD'length - 7;   -- =217-7, TM limit

   subtype T_SW_PARAM_INT_ID   is INTEGER range 0..SW_PARAMS_INT_NO-1;
   subtype T_SW_PARAM_FLOAT_ID is INTEGER range 0..SW_PARAMS_FLOAT_NO-1;

   subtype T_SW_PARAM_INT is CCT.T_UNSIGNED_16;

   type T_SW_PARAM_FLOAT is record
      W1,W2: CCT.T_UNSIGNED_16;
   end record;

   type T_SW_PARAMS_INT_FIELD   is array(T_SW_PARAM_INT_ID) of T_SW_PARAM_INT;
   type T_SW_PARAMS_FLOAT_FIELD is array(T_SW_PARAM_FLOAT_ID) of T_SW_PARAM_FLOAT;

   subtype T_DFEE_MEM_ADDRESS   is CCT.T_UNSIGNED_16 range 16#8000#..16#FFFF#;

   subtype T_MEM_BLOCK is IASW_COMMON.T_ARRAY_OF_WORDS;  -- generic type

   type T_MEM_BLOCK_SPEC is record
      ADDR   : T_DFEE_MEM_ADDRESS;
      LENGTH : INTEGER;
   end record;

   -- naming conventions used:
   -- Acc = Delta = Dv
   -- Drift = Cathode = Vc
   type T_DFEE_HK is record
      HK_CYCLE_COUNTER,
      DFEE_STATE,
      HW_STATUS,     -- 0..7: LowLevelDiscr.  8..15: Anode Config.
      HV_ACC,        -- 0..3: -,  4..15: Acc.monitoring
      HV_DRIFT,      -- 0..3: -,  4..15: Drift.monitoring
      HV_SETTINGS,   -- 0..7: ?,  8..15: ?
      EV_TRIGGERS,
      EV_ACCEPTED,
      EV_REJ_GREY_FILTER,
      EV_REJ_SHORT_BUFFER,
      EV_REJ_FIFO_FULL,
      EV_REJ_UPPER_TRESHOLD,
      EV_REJ_VETO_SIGNAL,
      EV_REJ_LOW_RISETIME,
      EV_REJ_HIGH_RISETIME,
      EV_REJ_TOO_MANY_HITS_BACK_PLANE,
      EV_REJ_TOO_MANY_HITS_CATHODE_PLANE,
      EV_REJ_TOO_HIGH_SIGNAL_BACK_PLANE,
      EV_REJ_TOO_LOW_SIGNAL_BACK_PLANE,
      EV_REJ_TOO_HIGH_SIGNAL_CATHODE_PLANE,
      EV_REJ_TOO_LOW_SIGNAL_CATHODE_PLANE,
      CAL_SPECT_PARTITION  : CCT.T_UNSIGNED_16;
      CALIBRATION_SPECTRUM : IASW_COMMON.T_ARRAY_OF_WORDS(1..32); --T_CALIBRATION_SPECTRUM;
      EV_REJ_WRAPS         : CCT.T_UNSIGNED_16;
      SPARE                : IASW_COMMON.T_ARRAY_OF_WORDS(1..1);
   end record;

   type T_DFEE_STATUS is record
      LAST_HW_COMMAND,
      LAST_CHG_PARAM_ID,
      ERROR_NO,
      HK_CYCLE_COUNTER,
      TIMER
         : CCT.T_UNSIGNED_16;
   end record;

   type T_DIAGNOSTIC_HK is record
      DFEE_STATE : T_DFEE_STATE;
   end record;

   type T_DFEE_DIAGNOSTIC is record
      TX_TOTAL_NO,     -- Number of commands sent do DFEE
      RX_ERROR_NO,     -- Number of errors detected in CSSW
      RX_CRC_NO,       -- Number of CRC errors in received blocks
      RX_TIMEOUT_NO,   -- Number of cases of no response
      RX_FAILURE_NO    -- Number of aborted communication requests
         : CCT.T_UNSIGNED_16;
   end record;

   type T_HW_SETTINGS is record
      ANODE_SETTINGS,
      DISCRIM_LOW_LEVEL,
      HV_ACC_SET,
      HV_DRIFT_SET
         : CCT.T_UNSIGNED_16;
   end record;


   --------------------------------------------------------------------
   -- Local types and data, made global because of possible patching --
   --------------------------------------------------------------------

   -- hardware characteristics of the channel
   LSL_CHANNEL         : constant CIT.T_LOW_SPEED_LINE_CHANNEL := 0; --0..3
   --LSL_BAUD_RATE       : constant CIT.T_BAUD_RATE := KBPS_04; --KBPS_02..KBPS_64
   --the above constant is not used.  instead procedure SET_LSL_PARAMS is called
   --with a parameter defining baud rate.
   LSL_TRANSMITTER     : constant CIT.T_TRANSMITTER_ADDRESS := 0; -- 1 octet

   -- s/w parameters of the communication procedure
   LSL_TIMEOUT            : constant CIT.T_RTC_TICKS := 12;
   LSL_MAX_REPT_NO        : constant INTEGER         := 3;
   LSL_MAX_LONG_TRANSFER  : constant INTEGER         := 230;
   LSL_MAX_SHORT_TRANSFER : constant INTEGER         := 6;

   -- Indicators of blocks sent by DFEE on LSL
   ANS_CODE_RQ_ACK          : constant CCT.T_UNSIGNED_16 := 16#F00F#;
   ANS_CODE_HK              : constant CCT.T_UNSIGNED_16 := 16#7777#;
   ANS_CODE_STATUS          : constant CCT.T_UNSIGNED_16 := 16#0F0F#;
   ANS_CODE_MEMORY_DUMP     : constant CCT.T_UNSIGNED_16 := 16#0FF0#;
   ANS_CODE_MEMORY_CRC      : constant CCT.T_UNSIGNED_16 := 16#F7F7#;
   ANS_CODE_PARAM_INT       : constant CCT.T_UNSIGNED_16 := 16#1357#;
   ANS_CODE_PARAM_FLOAT     : constant CCT.T_UNSIGNED_16 := 16#147A#;
   ANS_CODE_PAR_INT_FIELD   : constant CCT.T_UNSIGNED_16 := 16#BC01#;
   ANS_CODE_PAR_FLOAT_FIELD : constant CCT.T_UNSIGNED_16 := 16#BCEF#;

   -- Indicators of commands sent by DPE to DFEE on LSL
   COMM_CODE_SET_STATE      : constant CCT.T_UNSIGNED_16 := 16#1234#;
   COMM_CODE_HW_COMMAND     : constant CCT.T_UNSIGNED_16 := 16#AAAA#;
   COMM_CODE_CPU_COMMAND    : constant CCT.T_UNSIGNED_16 := 16#1608#;
   COMM_CODE_PARAM_INT_CHG  : constant CCT.T_UNSIGNED_16 := 16#8642#;
   COMM_CODE_PARAM_FLOAT_CHG: constant CCT.T_UNSIGNED_16 := 16#8765#;
   COMM_CODE_PARAM_INT_RQ   : constant CCT.T_UNSIGNED_16 := 16#FDB9#;
   COMM_CODE_PARAM_FLOAT_RQ : constant CCT.T_UNSIGNED_16 := 16#FEDC#;
   COMM_CODE_PAR_FIELD_RQ   : constant CCT.T_UNSIGNED_16 := 16#BBBB#;
   COMM_CODE_SET_GREY_FILTER: constant CCT.T_UNSIGNED_16 := 16#2468#;
   COMM_CODE_MEMORY_LOAD    : constant CCT.T_UNSIGNED_16 := 16#ABCD#;
   COMM_CODE_MEMORY_DUMP_RQ : constant CCT.T_UNSIGNED_16 := 16#CADB#;
   COMM_CODE_MEMORY_CRC_RQ  : constant CCT.T_UNSIGNED_16 := 16#7F7F#;
   COMM_CODE_STATUS_RQ      : constant CCT.T_UNSIGNED_16 := 16#2323#;
   COMM_CODE_HK_RQ          : constant CCT.T_UNSIGNED_16 := 16#4711#;
   COMM_CODE_ENTAB_1        : constant CCT.T_UNSIGNED_16 := 16#4644#;
   COMM_CODE_ENTAB_2        : constant CCT.T_UNSIGNED_16 := 16#4645#;
   COMM_CODE_EXEC_ENTAB     : constant CCT.T_UNSIGNED_16 := 16#4646#;

   -- Codes of Request Acknowledge Status
   RA_CODE_OK               : constant CCT.T_UNSIGNED_16 := 0;
   RA_CODE_CRC_ERROR        : constant CCT.T_UNSIGNED_16 := 1;
   RA_CODE_RQ_UNKNOWN       : constant CCT.T_UNSIGNED_16 := 2;
   RA_CODE_RQ_NOT_VALID     : constant CCT.T_UNSIGNED_16 := 3;
   RA_CODE_ID_NOT_VALID     : constant CCT.T_UNSIGNED_16 := 4;
   RA_CODE_PARAM_NOT_VALID  : constant CCT.T_UNSIGNED_16 := 5;
   RA_CODE_BUSY             : constant CCT.T_UNSIGNED_16 := 6;

   -- layout and size of parameters field in DFEE's RAM
   SW_PARAMS_INT_ADDR  : constant T_DFEE_MEM_ADDRESS := 16#D500#;
   SW_PARAMS_FLOAT_ADDR: constant T_DFEE_MEM_ADDRESS := 16#D600#;
   ENERGY_TABLE_ADDR   : constant T_DFEE_MEM_ADDRESS := 16#AC00#;

   K_HV_OFF_RELAY_ID   : constant CIT.T_RELAY_ID := 0;

   -- DFEE's context
   type T_DFEE_CONTEXT is record
      STATE  : T_DFEE_STATE;
      ST_PAR : CCT.T_UNSIGNED_16;
      SWPI   : T_SW_PARAMS_INT_FIELD;   -- SW int parameters of DFEE
      SWPF   : T_SW_PARAMS_FLOAT_FIELD; -- SW float parameters of DFEE
      SWP_VALID : BOOLEAN;              -- if SWPI and SWPF are valid
      HWST   : T_HW_SETTINGS;           -- actual HW settings
      HWTC   : T_HW_SETTINGS;           -- HW settings commanded from ground
      CPU    : CCT.T_UNSIGNED_16;       -- 8/16MHz, 0/1 wait states
      -- it is probably useless to store CPU state, because DFEE changes it also itself
      -- under conditions not fully understood (by GJ).
      HV_IS_ON : BOOLEAN;
      LV_IS_ON : BOOLEAN;
      DFEE_ACCESSIBLE : BOOLEAN;
      DFEE_HEETING_TIME : CCT.T_UNSIGNED_16;
      DFEE_HEETED : BOOLEAN;

      DFEE_LIVED_AT_LEAST_ONCE : BOOLEAN;  -- set after recpt. of LSL, never reset
      --in ver.1.63 moved here from IASW_STATE

      LSL_DEAD : BOOLEAN;
      LSL_POSSIBLY_DEAD : BOOLEAN;
      ENTAB  : IASW_COMMON.T_COMPRESSED_ENERGY_TABLE;
      SWPI_CRC : CCT.T_UNSIGNED_16;
      SWPF_CRC : CCT.T_UNSIGNED_16;
      COD1_CRC : CCT.T_UNSIGNED_16;
      COD2_CRC : CCT.T_UNSIGNED_16;
   end record;


   -- Format of blocks transmited via LSL

   -- the two constants below can take value 2, or 0.
   -- if another value is desired, changes should also be made
   -- in the procedures SET_LSL_PADDING1 and SET_LSL_PADDING2.
   K_P1_LEN : constant := 2;   -- header padding lenght
   K_P2_LEN : constant := 2;   -- trailer padding length
   K_PADD_LEN : constant := K_P1_LEN + K_P2_LEN;

   type T_LSL_QUESTION is record
      INDICATOR,
      PARAMETER,
      CRC               : CCT.T_UNSIGNED_16;
   end record;

   type T_LSL_COMMAND1 is record
      INDICATOR,
      COMMAND,
      CRC               : CCT.T_UNSIGNED_16;
   end record;

   type T_LSL_COMMAND2 is record
      INDICATOR,
      COMMAND,
      PARAMETER,
      CRC               : CCT.T_UNSIGNED_16;
   end record;

   subtype T_HALF_ENTAB is IASW_COMMON.T_ARRAY_OF_WORDS(1..128);

   type T_LSL_COMM_ENTAB is record
      INDICATOR         : CCT.T_UNSIGNED_16;
      HALF_ENTAB        : T_HALF_ENTAB;
      CRC               : CCT.T_UNSIGNED_16;
   end record;

   type T_LSL_COMM_SW_PARAM_INT is record
      INDICATOR         : CCT.T_UNSIGNED_16;
      PARAM_ID          : CCT.T_UNSIGNED_16;
      PARAM_VAL         : T_SW_PARAM_INT;
      CRC               : CCT.T_UNSIGNED_16;
   end record;

   type T_LSL_ANS_SW_PARAM_INT is record
      INDICATOR         : CCT.T_UNSIGNED_16;
      PARAM_ID          : CCT.T_UNSIGNED_16;
      PARAM_VAL         : T_SW_PARAM_INT;
      CRC               : CCT.T_UNSIGNED_16;
   end record;

   type T_LSL_COMM_SW_PARAM_FLOAT is record
      INDICATOR         : CCT.T_UNSIGNED_16;
      PARAM_ID          : CCT.T_UNSIGNED_16;
      PARAM_VAL         : T_SW_PARAM_FLOAT;
      CRC               : CCT.T_UNSIGNED_16;
   end record;

   type T_LSL_ANS_SW_PARAM_FLOAT is record
      INDICATOR         : CCT.T_UNSIGNED_16;
      PARAM_ID          : CCT.T_UNSIGNED_16;
      PARAM_VAL         : T_SW_PARAM_FLOAT;
      CRC               : CCT.T_UNSIGNED_16;
   end record;

   type T_LSL_ANS_SW_PARAMS_INT_FIELD is record
      INDICATOR         : CCT.T_UNSIGNED_16;
      LENGTH            : CCT.T_UNSIGNED_16;
      PAR_FIELD         : T_SW_PARAMS_INT_FIELD;
      CRC               : CCT.T_UNSIGNED_16;
   end record;

   type T_LSL_ANS_SW_PARAMS_FLOAT_FIELD is record
      INDICATOR         : CCT.T_UNSIGNED_16;
      LENGTH            : CCT.T_UNSIGNED_16;
      PAR_FIELD         : T_SW_PARAMS_FLOAT_FIELD;
      CRC               : CCT.T_UNSIGNED_16;
   end record;

   type T_LSL_COMM_GET_MEMORY is record
      INDICATOR         : CCT.T_UNSIGNED_16;
      MEM_BLOCK_HDR     : T_MEM_BLOCK_SPEC;
      CRC               : CCT.T_UNSIGNED_16;
   end record;

   type T_LSL_ANS_RQACK is record
      INDICATOR,
      RA_STATUS_CODE,
      CRC               : CCT.T_UNSIGNED_16;
   end record;

   type T_LSL_ANS_MEMORY is record
      INDICATOR         : CCT.T_UNSIGNED_16;
      MEM_BLOCK_HDR     : T_MEM_BLOCK_SPEC;
      MEM_BLOCK         : T_MEM_BLOCK(1..K_MAX_DUMP_MEM_BLK_LEN);
      CRC               : CCT.T_UNSIGNED_16;
   end record;
   K_LSL_ANS_MEMORY_FRAME_LEN : constant := 4;
   -- = total minus MEM_BLOCK length

   type T_LSL_COMM_LOAD_MEMORY is record
      INDICATOR         : CCT.T_UNSIGNED_16;
      MEM_BLOCK_HDR     : T_MEM_BLOCK_SPEC;
      MEM_BLOCK         : T_MEM_BLOCK(1..K_MAX_LOAD_MEM_BLK_LEN);
      CRC               : CCT.T_UNSIGNED_16;
   end record;
   K_LSL_COMM_MEMORY_FRAME_LEN : constant := 4;  -- +K_PADD_LEN;
   -- = total minus MEM_BLOCK length

   type T_LSL_ANS_MEMORY_CRC is record
      INDICATOR         : CCT.T_UNSIGNED_16;
      MEM_BLOCK_HDR     : T_MEM_BLOCK_SPEC;
      MEMORY_CRC        : CCT.T_UNSIGNED_16;
      CRC               : CCT.T_UNSIGNED_16;
   end record;

   type T_LSL_ANS_HK_BLOCK is record
      INDICATOR         : CCT.T_UNSIGNED_16;
      HK_BLOCK          : T_DFEE_HK;
      CRC               : CCT.T_UNSIGNED_16;
   end record;

   type T_LSL_ANS_STATUS is record
      INDICATOR         : CCT.T_UNSIGNED_16;
      STATUS            : T_DFEE_STATUS;
      CRC               : CCT.T_UNSIGNED_16;
   end record;

   -- Type of result of the local procedure LSL_TX_RX
   type T_LSL_RESULT is (
      LSL_RESULT_OK,
      LSL_RESULT_ERROR,
      LSL_RESULT_TIMEOUT
   );

   -- local data of the package

   CONTEXT           : T_DFEE_CONTEXT;
   DFEE_DIAGNOSTIC   : T_DFEE_DIAGNOSTIC;
   SETUP_IN_DT_CNT   : INTEGER;  --counter of occurencies SETUP when in active state.
   SW_PARAM_OBT      : CIT.T_OBT;     --OBT of the last change of SW param.
   LSL_DIAG_CODE     : CCT.T_UNSIGNED_16;  --id# of the event or exception from CSSW
   LSL_SEMAPHORE : CCT.T_UNSIGNED_16 := 0;
   -- these two _ID are forced to be integer to facilitate use in PUT_OEM()
   COMM_ID, ANS_ID   : INTEGER;  --indicators of the last command and answer
   -- general purpose mem. load/dump buffer, allocated here to save stack
   GL_MEM_BLOCK_HDR : T_MEM_BLOCK_SPEC;
   GL_MEM_BLOCK     : T_MEM_BLOCK(1..K_MAX_DUMP_MEM_BLK_LEN);
   -- these buffers are placed here to save stack space
   LSL_ANS_MEMORY : T_LSL_ANS_MEMORY;
   LSL_RQ_LOAD_MEMORY : T_LSL_COMM_LOAD_MEMORY;
   LSL_SW_PARAMS_INT_FIELD : T_LSL_ANS_SW_PARAMS_INT_FIELD;
   LSL_SW_PARAMS_FLOAT_FIELD : T_LSL_ANS_SW_PARAMS_FLOAT_FIELD;
   LSL_ANS_HK_BLOCK : T_LSL_ANS_HK_BLOCK;
   LSL_RQ_ENTAB : T_LSL_COMM_ENTAB;

   DFEE_STATE_IN_HK_VALID : BOOLEAN;
   -- this is set on reception of HK, and cleared on any state change.

   --USE_CSSW_17 : constant BOOLEAN := FALSE;
   --deleted in ver.1.62 (this constant no more used)

   ----------------------------------------------------------------
   -- Local procedures, made global because of possible patching --
   ----------------------------------------------------------------

   procedure MODIFY_EXPECTED_STATE_IF_NEEDED;
   procedure DFEE_ANSWERED;
   procedure DFEE_DOES_NOT_ANSWER;
   procedure ENTER_LSL_CRITSEC;
   procedure LEAVE_LSL_CRITSEC;
   function GET_CRC_IN_MY_AS(
      ADDRESS : in CIT.T_LOGICAL_ADDRESS;
      LENGTH  : in INTEGER ) return CCT.T_UNSIGNED_16;
   procedure ADD_PATCH(
      HDR  : in T_MEM_BLOCK_SPEC;
      DATA : in T_MEM_BLOCK );
   procedure GET_PATCH(
      NUM  : in INTEGER;
      HDR  : out T_MEM_BLOCK_SPEC;
      DATA : out T_MEM_BLOCK );
   procedure SET_LSL_PADDING1(
      BUF : out IASW_COMMON.T_ARRAY_OF_WORDS;
      J   : in out INTEGER );
   procedure SET_LSL_PADDING2(
      BUF : out IASW_COMMON.T_ARRAY_OF_WORDS;
      J   : in out INTEGER );
   procedure LSL_TX_RX (
      TX_BUF : in CIT.T_LOGICAL_ADDRESS;
      TX_LEN : in INTEGER;
      RX_BUF : in CIT.T_LOGICAL_ADDRESS;
      RX_LEN : in INTEGER;
      TIMEOUT : in CIT.T_RTC_TICKS;
      REAL_RX_LEN : out INTEGER;
      RESULT : out T_LSL_RESULT );
   procedure DPE_DFEE_DIALOG(
      COMMAND     : in CIT.T_LOGICAL_ADDRESS;
      COMMAND_LEN : in INTEGER;
      ANSWER      : in CIT.T_LOGICAL_ADDRESS;
      ANSWER_LEN  : in INTEGER := 0;
      ANSWER_ID   : in CCT.T_UNSIGNED_16 := 0;
      OEM_RA6     : in BOOLEAN := TRUE );
   procedure TEST_LSL_DIALOG(
      PARAM_VAL: in IASW_COMMON.T_TC_PARAM_VAL;
      PARAM_LEN: in IASW_COMMON.T_SW_PARAM_LEN );
   procedure UPDATE_IF_OVERLAYS(
      MEM_BLOCK_HDR : in T_MEM_BLOCK_SPEC;
      MEM_BLOCK     : in T_MEM_BLOCK;
      TARGET_DFEE_ADDR : in T_DFEE_MEM_ADDRESS;
      TARGET_DPE_ADDR  : in SYSTEM.ADDRESS;
      TARGET_LEN    : in INTEGER );
   procedure READ_CRC(COMPARE,SAVE : in BOOLEAN);
   procedure CLEAR_DIAGNOSTIC;
   procedure SET_DEFAULT_CONTEXT;
   procedure SET_LSL_MODE;


   -----------------------
   -- Global procedures --
   -----------------------

   procedure SET_LSL_PARAMS (
      CLK_FREQ_ID   : in CCT.T_UNSIGNED_16;
      BDR_FACTOR_ID : in CCT.T_UNSIGNED_16 := 1 );

   procedure CONNECT_TO_LSL;
   -- To be used by processes which will send commands to DFEE.

   function GET_STATE return T_DFEE_STATE;
   -- Return State code stored in IASW (no LSL dialog involved).

   procedure GET_HW_SETTINGS( HWS : out T_HW_SETTINGS );
   -- Returns actual HV, Anode and Discrim. level settings (no LSL dialog involved).

   procedure GET_LAST_COMMANDED_HW_SETTINGS( HWS : out T_HW_SETTINGS );
   -- Returns the same data as GET_HW_SETTINGS, but those commanded with TC.
   -- Command "HV on" sets commanded acc and drift HV to minimal defaults.

   procedure GET_HK( HK : out T_DFEE_HK );

   procedure GET_STATUS( STATUS : out T_DFEE_STATUS );

   procedure DUMP_MEMORY (
      MEM_BLOCK_HDR  : in  T_MEM_BLOCK_SPEC;
      MEM_BLOCK      : out T_MEM_BLOCK );

   procedure GET_MEMORY_CRC (
      MEM_BLOCK_HDR  : in  T_MEM_BLOCK_SPEC;
      CRC            : out CCT.T_UNSIGNED_16 );

   procedure LOAD_MEMORY (
      MEM_BLOCK_HDR : in T_MEM_BLOCK_SPEC;
      MEM_BLOCK     : in T_MEM_BLOCK;
      UPD_CONTEXT   : in BOOLEAN := TRUE );

   procedure GET_SW_PARAM_INT (
      PARAM_ID    : in CCT.T_UNSIGNED_16;
      PARAM_VALUE : out T_SW_PARAM_INT );

   procedure GET_SW_PARAM_FLOAT (
      PARAM_ID    : in CCT.T_UNSIGNED_16;
      PARAM_VALUE : out T_SW_PARAM_FLOAT );

   procedure GET_SW_INT_PARAMS(
      SWPI    : out T_SW_PARAMS_INT_FIELD;
      USE_SEM : in BOOLEAN := TRUE );

   procedure GET_SW_FLOAT_PARAMS(
      SWPF    : out T_SW_PARAMS_FLOAT_FIELD;
      USE_SEM : in BOOLEAN := TRUE );

   procedure CHG_SW_PARAM_INT (
      PARAM_ID    : in CCT.T_UNSIGNED_16;
      PARAM_VALUE : in T_SW_PARAM_INT );

   procedure CHG_SW_PARAM_FLOAT (
      PARAM_ID    : in CCT.T_UNSIGNED_16;
      PARAM_VALUE : in T_SW_PARAM_FLOAT );

   procedure CHG_CPU_SETTING (
      VALUE       : in CCT.T_UNSIGNED_16;
      OEM_RA6     : in BOOLEAN := TRUE );

   procedure CHG_HW_SETTING (
      COMMAND     : in CCT.T_UNSIGNED_16;
      PARAM       : in CCT.T_UNSIGNED_16;
      OEM_RA6     : in BOOLEAN := TRUE );

   -- procedures derived from CHG_HW_SETTING()
   procedure HV_READY(
      OEM_RA6 : in BOOLEAN := TRUE );
   procedure HV_ON(
      OEM_RA6 : in BOOLEAN := TRUE );
   procedure HV_OFF(
      OEM_RA6 : in BOOLEAN := TRUE );
   procedure SET_ANODE(
      PARAM : in CCT.T_UNSIGNED_16;
      OEM_RA6 : in BOOLEAN := TRUE );
   procedure SET_DISCRIM_LOW_LEVEL(
      PARAM : in CCT.T_UNSIGNED_16;
      OEM_RA6 : in BOOLEAN := TRUE );
   procedure SET_HV_ACC(
      PARAM : in CCT.T_UNSIGNED_16;
      OEM_RA6 : in BOOLEAN := TRUE );
   procedure SET_HV_DRIFT(
      PARAM : in CCT.T_UNSIGNED_16;
      OEM_RA6 : in BOOLEAN := TRUE );

   function HV_IS_ON return BOOLEAN;

   function LSL_DEAD return BOOLEAN;

   function DFEE_HEETED return BOOLEAN;

   function DFEE_LIVED_AT_LEAST_ONCE return BOOLEAN;
   --new since ver.1.63

   procedure SEND_HV_OFF_RELAY_SIGNAL;

   procedure LOW_VOLTAGE_IS_OFF;
   -- it is called on detection of low voltage failure.

   procedure LOW_VOLTAGE_IS_ON;
   -- it is called on detection of correct low voltage.

   procedure BCP4_TICK;

   procedure CHECK_STATE(
      HK_STATE : in T_DFEE_STATE;
      DATA_TAKING_STOPPED : out BOOLEAN );

   procedure ACCEPT_STATE( STATE : in T_DFEE_STATE );

   procedure SET_STATE (
      STATE         : in T_DFEE_STATE;
      PARAM         : in CCT.T_UNSIGNED_16 := 0;
      OEM_RA6       : in BOOLEAN := TRUE );

   procedure GOTO_STATE(STATE : in T_DFEE_STATE);
   -- changes to the state required regardless of the current state.

   procedure SET_GREY_FILTER (
      GF_ID         : in CCT.T_UNSIGNED_16 );

   procedure SET_DEFAULT_ENERGY_TABLE(
      ENTAB         : in IASW_COMMON.T_COMPRESSED_ENERGY_TABLE );

   procedure SET_ENERGY_TABLE(
      ENTAB         : in IASW_COMMON.T_COMPRESSED_ENERGY_TABLE;
      UPD_CONTEXT   : in BOOLEAN := TRUE;
      USE_SEM       : in BOOLEAN := TRUE );

   procedure EXEC_ENERGY_TABLE(
      USE_SEM       : in BOOLEAN := TRUE );
   -- this is to be used after loading new energy table

   procedure SAVE_CONTEXT;
   -- To be used before shutdown, or on request TC(5,3).

   procedure LOAD_CONTEXT;
   -- To be used in automatic recovering or for
   -- performing TC(5,3)-Load DFEE Context.
   -- It can be successful only if before SAVE_CONTEXT was issued

   function REDUCED_HV(
      HV    : in CCT.T_UNSIGNED_16;
      HVMIN : in CCT.T_UNSIGNED_16;
      HVDELTA : in CCT.T_UNSIGNED_16) return CCT.T_UNSIGNED_16;

   procedure SET_SW_PARAM(
      PARAM_ID:  in IASW_COMMON.T_FID;
      PARAM_VAL: in IASW_COMMON.T_TC_PARAM_VAL;
      PARAM_LEN: in IASW_COMMON.T_SW_PARAM_LEN);

   procedure GET_SW_PARAM(
      PARAM_ID:  in  IASW_COMMON.T_FID;
      PARAM_VAL: out IASW_COMMON.T_TM_PARAM_VAL;
      PARAM_LEN: out IASW_COMMON.T_SW_PARAM_LEN);

   procedure GET_DPE_STATE(DS : out IASW_COMMON.T_DPE_STATE);

   procedure GET_HK_DIAG(
      FLAGS : out IASW_COMMON.T_16_BITS;
      DIAG  : out T_DIAGNOSTIC_HK);

   procedure GET_AND_CLEAR_DIAGNOSTIC_DATA( DIAG : out T_DFEE_DIAGNOSTIC );

   procedure CONSTRUCTOR;

end IASW_DFEE;
