with CSSW_COMMON_TYPES;  use CSSW_COMMON_TYPES;
with CSSW_IF_TYPES;      use CSSW_IF_TYPES;
with IASW_COMMON;
with IASW_BUFFER;
with IASW_MODES;

package IASW_GF is

   package CIT renames CSSW_IF_TYPES;
   package CCT renames CSSW_COMMON_TYPES;

   subtype T_GREY_FILTER_INDICATOR is CCT.T_UNSIGNED_16 range 0..31;

   GREY_FILTER_0   : constant T_GREY_FILTER_INDICATOR :=
			      T_GREY_FILTER_INDICATOR'first;
   GREY_FILTER_100 : constant T_GREY_FILTER_INDICATOR :=
			      T_GREY_FILTER_INDICATOR'last;

   --------------------------------------------------------------------
   -- Local types and data, made global because of possible patching --
   --------------------------------------------------------------------

   GREY_FILTER_WANTED : T_GREY_FILTER_INDICATOR; -- set by the program
   AUTOMATIC_GREY_FILTER_SELECTION : BOOLEAN;

   -- Default buffer levels in % (between 0 and 100).
   DEFAULT_EXTRA_LOW_LEVEL   : constant CCT.T_UNSIGNED_16 := 5;
   DEFAULT_NORMAL_LOW_LEVEL  : constant CCT.T_UNSIGNED_16 := 20;
   DEFAULT_NORMAL_HIGH_LEVEL : constant CCT.T_UNSIGNED_16 := 50;
   DEFAULT_EXTRA_HIGH_LEVEL  : constant CCT.T_UNSIGNED_16 := 80;

   -- Current buffer levels in % (between 0 and 100).
   EXTRA_LOW_LEVEL   : CCT.T_UNSIGNED_16;
   NORMAL_LOW_LEVEL  : CCT.T_UNSIGNED_16;
   NORMAL_HIGH_LEVEL : CCT.T_UNSIGNED_16;
   EXTRA_HIGH_LEVEL  : CCT.T_UNSIGNED_16;

   -- The buffer levels converted to the positions in buffer.
   EXTRA_LOW_EVENT_THRESHOLD   : IASW_BUFFER.T_BUFFER_INDEX;
   NORMAL_LOW_EVENT_THRESHOLD  : IASW_BUFFER.T_BUFFER_INDEX;
   NORMAL_HIGH_EVENT_THRESHOLD : IASW_BUFFER.T_BUFFER_INDEX;
   EXTRA_HIGH_EVENT_THRESHOLD  : IASW_BUFFER.T_BUFFER_INDEX;

   --
   PREVIOUS_BUFFER_LEVEL : IASW_BUFFER.T_BUFFER_INDEX;

   EVENTS_PER_TM_BLOCK  : CCT.T_UNSIGNED_16;  -- depending on the Im.Format
   TM_BLOCKS_PER_BCP1   : CCT.T_UNSIGNED_16;  -- from BCPKT
   ASS_TM_RATE          : CCT.T_UNSIGNED_16;  -- theoretical TM throughput (events / 8 sec)
   -- ASS_TM_RATE = TM_BLOCKS_PER_BCP1 * EVENTS_PER_TM_BLOCK


   ----------------------------------------------------------------
   -- Local procedures, made global because of possible patching --
   ----------------------------------------------------------------

   procedure SET_GREY_FILTER_ACCORDING_TO_EVENTS_RATES(
      GF : in out T_GREY_FILTER_INDICATOR );

   -----------------------
   -- Global procedures --
   -----------------------

   procedure SET_EVENTS_PER_TM_BLOCK(
     SCF : in IASW_MODES.T_SCIENCE_FORMAT;
     EVENT_SIZE : in IASW_COMMON.T_EVENT_SIZE );
   -- this is to be used whenever TASK_HSL decides about
   -- changing of format (prim./sec.)

   procedure SET_TM_BLOCKS_PER_BCP1( BLNO : in CCT.T_UNSIGNED_16 );
   -- this is to be used after reception of BCPKT

   procedure CALCULATE_BUFFER_THRESHOLDS;
   -- this is to be used after setting new EVENT_SIZE

   procedure INIT_GF;

   procedure SET_GF;

   procedure ANALYSE_GREY_FILTER;
   -- this is called periodicaly in TASK_HSL, it sends command
   -- to DFEE if needed

   procedure SET_BUFFER_LEVELS (
      PARAM_VAL : in IASW_COMMON.T_TC_PARAM_VAL;
      PARAM_LEN : in IASW_COMMON.T_SW_PARAM_LEN);
   -- called on reception of TC(5,3)

   procedure GET_BUFFER_LEVELS (
      PARAM_VAL : out IASW_COMMON.T_TM_PARAM_VAL;
      PARAM_LEN : out IASW_COMMON.T_SW_PARAM_LEN);
   -- called on reception of TC(5,4)

   procedure SET_GREY_FILTER (
      PARAM_VAL : in IASW_COMMON.T_TC_PARAM_VAL;
      PARAM_LEN : in IASW_COMMON.T_SW_PARAM_LEN);
   -- called on reception of TC(5,3), sends command to DFEE if needed.

   procedure GET_DPE_STATE(DS : out IASW_COMMON.T_DPE_STATE);

   procedure GET_HK_DIAG( FLAGS : out IASW_COMMON.T_16_BITS );

   procedure CONSTRUCTOR;

end IASW_GF;
