----------------------------------------------------------------------------
-- Module Name:        IASW_HK - package body
-- Project:            INTEGRAL, JEM-X, DPE
-- Author:             Dariusz Jaszkowski
-- Creation Date:      16.III.1998
-- Change History:
--    (Version/Date/Author/Description)
-- Function:
--    (Say what the component does)
--    Task HK - collects information to build HK packet, and sends
--    the packet to telemetry.
-- Interfaces:
--    (Define the inputs and outputs)
-- Dependencies:
--    (Describe the preconditions for using this component)
-- Processing:
--    (Summarise the processing using pseudo-code or PDL)
--    This package contains body of task HK.  The task is activated
--    periodically in task MAIN.  After completing the job task HK
--    terminates.
----------------------------------------------------------------------------

with IASW_COMMON;
with IASW_TIME;          use IASW_TIME;
with DPE_CSSW;
with IASW_DFEE;
with IASW_MRTU;
with IASW_HSL;
with IASW_TM;
with IASW_SC_OUT;
with IASW_STATE;
with IASW_BUFFER;
with IASW_MODES;
with IASW_PATCHES;
with IASW_GF;


package body IASW_HK is

   procedure CLEAR_IASW_HK is   -- local
   begin
       for I in IASW_HK_BLOCK'range loop
           IASW_HK_BLOCK(I) := 0;
       end loop;
       IASWHK.SPARE1 := 16#1111#;
       IASWHK.SPARE2 := 16#7777#;
       IASWHK.SPARE3 := 16#AAAA#;
       IASWHK.SPARE4 := 16#FFFF#;
   end CLEAR_IASW_HK;


   procedure CHECK_CONTINUITY_OF_DFEE_HK_CC( CC1,CC2 : CCT.T_UNSIGNED_16 ) is
   begin
      if IASW_COMMON.INC_MODULO_FFFF(CC1) /= CC2 then
         IASW_TM.PUT_OEM(IASW_TM.EV_DFEE_HK_CC,
           IASW_COMMON.CVT(CC1),
           IASW_COMMON.CVT(CC2));
      end if;
   end CHECK_CONTINUITY_OF_DFEE_HK_CC;


   procedure GET_DFEE_HK is   -- local
      PREV_DFEE_HK_VALID : BOOLEAN;
      PREV_DFEE_HK_CC : CCT.T_UNSIGNED_16;
      STOP : BOOLEAN;
   begin
      begin
         PREV_DFEE_HK_VALID := DFEEHK_VALID;
         PREV_DFEE_HK_CC := DFEEHK.HK_CYCLE_COUNTER;

         IASW_DFEE.GET_HK( DFEEHK );
         DFEEHK_VALID := TRUE;
         IASWHK.DFEE_HK := DFEEHK;

         IASW_TIME.SYNCHRONIZE_DFEE_WITH_OBT( DFEEHK.HK_CYCLE_COUNTER );
         IASW_HSL.NEW_HK_CC( DFEEHK.HK_CYCLE_COUNTER );
         IASW_DFEE.CHECK_STATE( DFEEHK.DFEE_STATE, STOP );
         if STOP then
            IASW_MODES.STOP_DATA_TAKING;
         end if;
         if PREV_DFEE_HK_VALID then
            CHECK_CONTINUITY_OF_DFEE_HK_CC(
               PREV_DFEE_HK_CC,
               DFEEHK.HK_CYCLE_COUNTER
            );
         end if;
      exception
         when others =>
            DFEEHK_VALID := FALSE;
      end;
   end GET_DFEE_HK;


   procedure MOVE_LAST_BCPKT_TO_HK is   -- local
      VALID : BOOLEAN;
      N : constant INTEGER := IASWHK.BCPKT'size/16;
      ZZ : array(1..N) of INTEGER;
         for ZZ use at IASWHK.BCPKT'address;
      -- ZZ is of the same length as IASWHK.BCPKT
   begin
      IASW_STATE.GET_LAST_BCPKT(IASWHK.BCPKT, VALID);
      if not VALID then
         for i in ZZ'range loop
            ZZ(i) := 0;
         end loop;
      end if;
   end MOVE_LAST_BCPKT_TO_HK;


   procedure DPE_STATE_TO_HK is   -- local
   begin
      IASW_MODES.  GET_DPE_STATE(IASWHK.DPE_STATE);
      IASW_STATE.  GET_DPE_STATE(IASWHK.DPE_STATE);
      IASW_SC_OUT. GET_DPE_STATE(IASWHK.DPE_STATE);
      IASW_HSL.    GET_DPE_STATE(IASWHK.DPE_STATE);
      IASW_DFEE.   GET_DPE_STATE(IASWHK.DPE_STATE);
      IASW_BUFFER. GET_DPE_STATE(IASWHK.DPE_STATE);
      IASW_GF.     GET_DPE_STATE(IASWHK.DPE_STATE);
      --IASW_PATCHES.GET_DPE_STATE(IASWHK.DPE_STATE);
      -- The above can't be used, because IASW_PATCHES works in AS14, and
      -- the below is used instead.
      IASWHK.DPE_STATE.MEMORY_PATCHES := IASW_PATCHES.GET_NUM_PATCHES;
   end DPE_STATE_TO_HK;


   procedure DIAGNOSTIC_TO_HK is   -- local
   begin
      IASW_COMMON.GET_HK_DIAG(IASWHK.DIAG.FLAGS, IASWHK.DIAG.COMMON);
      IASW_TM.    GET_HK_DIAG(IASWHK.DIAG.FLAGS, IASWHK.DIAG.TM);
      IASW_DFEE.  GET_HK_DIAG(IASWHK.DIAG.FLAGS, IASWHK.DIAG.DFEE);
      IASW_STATE. GET_HK_DIAG(IASWHK.DIAG.FLAGS, IASWHK.DIAG.STATE);
      IASW_BUFFER.GET_HK_DIAG(IASWHK.DIAG.FLAGS);
      IASW_GF.    GET_HK_DIAG(IASWHK.DIAG.FLAGS);
      IASW_HSL.   GET_HK_DIAG(IASWHK.DIAG.FLAGS);
   end DIAGNOSTIC_TO_HK;


   procedure GET_MRTU_HK is   -- local
      ANALOG_CHANNEL: constant array(IASW_COMMON.T_MRTU_INPUT) of CIT.T_ANALOG_CHANNEL := (
         IASW_COMMON.DET_PRESSURE_1  => CIT.ANALOG_ACQUISITION_8,
         IASW_COMMON.DET_PRESSURE_2  => CIT.ANALOG_ACQUISITION_9,
         IASW_COMMON.DET_TEMP_1      => CIT.ANALOG_ACQUISITION_10,
         IASW_COMMON.DET_TEMP_2      => CIT.ANALOG_ACQUISITION_11,
         IASW_COMMON.VOLTAGE_P5V_DIG => CIT.ANALOG_ACQUISITION_0,
         IASW_COMMON.CURRENT_P5V_DIG => CIT.ANALOG_ACQUISITION_1,
         IASW_COMMON.VOLTAGE_P5V_ANA => CIT.ANALOG_ACQUISITION_2,
         IASW_COMMON.VOLTAGE_M5V_ANA => CIT.ANALOG_ACQUISITION_3,
         IASW_COMMON.VOLTAGE_P12V    => CIT.ANALOG_ACQUISITION_4,
         IASW_COMMON.CURRENT_P12V    => CIT.ANALOG_ACQUISITION_5,
         IASW_COMMON.VOLTAGE_M12V    => CIT.ANALOG_ACQUISITION_6,
         IASW_COMMON.CURRENT_M12V    => CIT.ANALOG_ACQUISITION_7,
         IASW_COMMON.TEMP_1          => CIT.YSI_THERMISTOR_0,
         IASW_COMMON.TEMP_2          => CIT.YSI_THERMISTOR_1,
         IASW_COMMON.TEMP_3          => CIT.YSI_THERMISTOR_2,
         IASW_COMMON.TEMP_4          => CIT.YSI_THERMISTOR_3,
         IASW_COMMON.TEMP_5          => CIT.YSI_THERMISTOR_4,
         IASW_COMMON.TEMP_6          => CIT.YSI_THERMISTOR_5,
         IASW_COMMON.FIFO_FLAG       => CIT.ANALOG_ACQUISITION_12,
         IASW_COMMON.HV_PS_TEMP_1    => CIT.ANALOG_ACQUISITION_13,
         IASW_COMMON.HV_PS_TEMP_2    => CIT.ANALOG_ACQUISITION_14,
         IASW_COMMON.SPARE_ANALOG_CHANNEL => CIT.ANALOG_ACQUISITION_15
      );
   begin
      for I in IASW_COMMON.T_MRTU_INPUT loop
         begin
            MRTU_HK(I) := CCT.T_UNSIGNED_16(
              IASW_MRTU.GET_ANALOG_INPUT(ANALOG_CHANNEL(I)));
         exception
            when others =>
               MRTU_HK(I) := 16#100#;   -- marker of error
               -- OEM is generated in GET_ANALOG_INPUT()
         end;
      end loop;

      IASWHK.MRTU_HK := MRTU_HK;
   end GET_MRTU_HK;


   procedure PUT_HK_TM is   -- local
      LOCAL_IASW_HK_BLOCK : CIT.T_IASW_HK_DATA_FIELD;
   begin
      LOCAL_IASW_HK_BLOCK := IASW_HK_BLOCK;  -- CSSW wants to have it on stack.
      DPE_CSSW.PUT_IASW_HK_TM( LOCAL_IASW_HK_BLOCK );
   end PUT_HK_TM;


   procedure TASK_HK is
   --
   -- This task is activated by the TASK_MAIN 2 seconds before each BCP1.
   --
   begin
      IASW_MRTU.CONNECT_TO_MRTU;
      IASW_DFEE.CONNECT_TO_LSL;
      loop
         IASW_COMMON.I_AM_ALIVE( IASW_COMMON.TASK_HK_NO );

         CLEAR_IASW_HK;

         GET_MRTU_HK;
         IASW_STATE.ANALYSE_MRTU_DATA(MRTU_HK);
         -- in case of low voltage failure this will drop the DFEE state
         -- to STANDBY (NOT HEETED), thus preventing from reading of HK.

         if IASW_DFEE.DFEE_HEETED then
            GET_DFEE_HK;
         else
            DFEEHK_VALID := FALSE;
         end if;

         MOVE_LAST_BCPKT_TO_HK;
         DIAGNOSTIC_TO_HK;
         DPE_STATE_TO_HK;
         PUT_HK_TM;

         IASW_STATE.SET_WANTED_SHUTDOWN_LEVEL;

         DPE_CSSW.TERMINATE_TASK;
         --will be resumed again in 8 seconds.

      end loop;
   end TASK_HK;


   procedure CONSTRUCTOR is
   begin
      DFEEHK_VALID := FALSE;
      CLEAR_IASW_HK;
   end CONSTRUCTOR;


end IASW_HK;
