with SYSTEM;
with CSSW_COMMON_TYPES;  use CSSW_COMMON_TYPES;
with CSSW_IF_TYPES;      use CSSW_IF_TYPES;
with IASW_COMMON;
with IASW_DFEE;
with IASW_TM;
with IASW_STATE;

package IASW_HK is

   package CCT renames CSSW_COMMON_TYPES;
   package CIT renames CSSW_IF_TYPES;

   --------------------------------------------------------------------
   -- Local types and data, made global because of possible patching --
   --------------------------------------------------------------------

   -- telemetry house keeping block:

   type T_DIAGNOSTIC_HK_COLLECTION is record
      FLAGS  : IASW_COMMON.T_16_BITS;          -- 1
      TM     : IASW_TM.    T_DIAGNOSTIC_HK;    -- 1
      DFEE   : IASW_DFEE.  T_DIAGNOSTIC_HK;    -- 1
      STATE  : IASW_STATE. T_DIAGNOSTIC_HK;    -- 4
      COMMON : IASW_COMMON.T_DIAGNOSTIC_HK;    -- 4
   end record;

   type T_IASW_HK is record
      DPE_STATE : IASW_COMMON.T_DPE_STATE;
      SPARE1    : CCT.T_UNSIGNED_16;  --1111
      MRTU_HK   : IASW_COMMON.T_MRTU_HK;
      SPARE2    : CCT.T_UNSIGNED_16;  --7777
      DFEE_HK   : IASW_DFEE.T_DFEE_HK;
      SPARE3    : CCT.T_UNSIGNED_16;  --AAAA
      BCPKT     : IASW_COMMON.T_BROADCAST_PACKET;
      SPARE4    : CCT.T_UNSIGNED_16;  --FFFF
      -- End of official part of the HK TM
      DIAG      : T_DIAGNOSTIC_HK_COLLECTION;
      -- !!! Max total = CIT.K_IASW_HK_DATA_LEN=145 words
   end record;

   IASW_HK_BLOCK : CIT.T_IASW_HK_DATA_FIELD;
   IASWHK : T_IASW_HK;
      for IASWHK use at IASW_HK_BLOCK'address;
pragma If(((IASWHK'size+15)/16) > IASW_HK_BLOCK'length);
   error: IASW HK block definition too long!
pragma End if;

   -- dfee hk block:
   DFEEHK : IASW_DFEE.T_DFEE_HK;
   DFEEHK_VALID : BOOLEAN;

   MRTU_HK : IASW_COMMON.T_MRTU_HK;


   ----------------------------------------------------------------
   -- Local procedures, made global because of possible patching --
   ----------------------------------------------------------------

   procedure CLEAR_IASW_HK;
   procedure CHECK_CONTINUITY_OF_DFEE_HK_CC( CC1,CC2 : CCT.T_UNSIGNED_16 );
   procedure MOVE_LAST_BCPKT_TO_HK;
   procedure DPE_STATE_TO_HK;
   procedure DIAGNOSTIC_TO_HK;
   procedure PUT_HK_TM;
   procedure GET_MRTU_HK;
   procedure GET_DFEE_HK;

   -----------------------
   -- Global procedures --
   -----------------------

   procedure TASK_HK;
   procedure CONSTRUCTOR;

end IASW_HK;
