with CSSW_COMMON_TYPES;
with CSSW_IF_TYPES;
with IASW_COMMON;

package IASW_MAIN is

   package CIT renames CSSW_IF_TYPES;
   package CCT renames CSSW_COMMON_TYPES;

   --group of exceptions generating corresponding OEMs - EV_TC_REJ_xxxx
   EX_TYPE_SUBTYPE,    --command rejected at front-end
   EX_TID_FID,         --command rejected at front-end
   EX_MID,             --command rejected at front-end
   EX_ADDRESS,         --command rejected at front-end
   EX_LENGTH,          --command rejected at front-end
   EX_DATA,            --command rejected at front-end
   EX_BUSY,            --queue of commands full (not used)
   EX_SAFETY,          --command not allowed because of shutdown mode
   EX_STATE,           --state not allowed because of shutdown mode
   EX_EXEC_FAILED      --execution of the command initiated but failed
      : exception;

   -- TIDs for TC(5,x):
   TID_DFEE :   constant IASW_COMMON.T_TID := 0;
   TID_MAIN :   constant IASW_COMMON.T_TID := 1;
   TID_HSL :    constant IASW_COMMON.T_TID := 3;
   TID_SC_OUT : constant IASW_COMMON.T_TID := 4;

   -- diagnostic data record:
   type T_TC_DIAGNOSTIC is record
      TC_COUNT,               -- received telecommand counter:
      TC_REJECTED_COUNT,      -- rejected telecommand counter:
      TC_TIMEOUT_COUNT,       -- telecommand read timeout counter:
      BCP1_TIMEOUT_COUNT,     -- BCP1 timeout counter:
      BCP1_ERROR_COUNT,       -- BCP1 error counter:
      RTC_TIMEOUT_COUNT       -- RTC timeout counter:
         : CCT.T_UNSIGNED_16;
   end record;

   --------------------------------------------------------------------
   -- Local types and data, made global because of possible patching --
   --------------------------------------------------------------------

   -- amount of time the next IASW TC ICB check will be delayed
   -- when the buffer is empty:
   ICB_READ_TIMEOUT : constant CIT.T_RTC_TICKS := 100;

   BCP1_TIMEOUT : constant CIT.T_RTC_TICKS := 66;
   BCP4_TIMEOUT : constant CIT.T_RTC_TICKS := 10;

   -- MID_DFEE - the only allowed value for MID.
   MID_DFEE : constant CCT.T_UNSIGNED_16 := 255;

   -- diagnostic data:
   MAIN_DIAGNOSTIC : T_TC_DIAGNOSTIC;

   -- IASW diagnostic report period expressed in terms of a number of BCP1
   -- cycles (0=automatic generation is disabled):
   IASW_DIAGNOSTIC_PERIOD : CCT.T_UNSIGNED_16;

   -- TC description reported in OEM-s
   TC_OEM_DATA_FIELD2, TC_OEM_DATA_FIELD3 : CCT.T_UNSIGNED_16;


   ----------------------------------------------------------------
   -- Local procedures, made global because of possible patching --
   ----------------------------------------------------------------

   procedure TC_MODE_TRANSITION(TC_DATA: in CIT.T_TC_APPL_DATA_FIELD);
   procedure TC_LOAD_TASK_PARAMS(TC_DATA: in CIT.T_TC_APPL_DATA_FIELD);
   procedure TC_REPORT_TASK_PARAMS(TC_DATA: in CIT.T_TC_APPL_DATA_FIELD);
   procedure TC_LOAD_MEMORY(TC_DATA: in CIT.T_TC_APPL_DATA_FIELD);
   procedure TC_DUMP_MEMORY(TC_DATA: in CIT.T_TC_APPL_DATA_FIELD);
   procedure TC_GET_MEMORY_CRC(TC_DATA: in CIT.T_TC_APPL_DATA_FIELD);
   procedure TC_BROADCAST_PACKET(TC_DATA: in CIT.T_TC_APPL_DATA_FIELD);
   procedure PREPARE_TC_OEM_INFO(TC : in CIT.T_TCSP);
   procedure EXECUTE_TC(TC : in CIT.T_TCSP);
   procedure WAIT_FOR_VALID_TC(TCSP : in out CIT.T_OPT_TCSP);
   procedure CLEAR_DIAGNOSTIC_DATA;

   -----------------------
   -- Global procedures --
   -----------------------

   procedure TASK_MAIN;
   procedure TASK_TC;
   procedure GET_AND_CLEAR_DIAGNOSTIC_DATA(DATA: out T_TC_DIAGNOSTIC);
   procedure SET_SW_PARAM(
      PARAM_ID : in IASW_COMMON.T_FID;
      PARAM_VAL: in IASW_COMMON.T_TC_PARAM_VAL;
      PARAM_LEN: in IASW_COMMON.T_SW_PARAM_LEN);
   procedure GET_SW_PARAM(
      PARAM_ID : in  IASW_COMMON.T_FID;
      PARAM_VAL: out IASW_COMMON.T_TM_PARAM_VAL;
      PARAM_LEN: out IASW_COMMON.T_SW_PARAM_LEN);
   procedure CONSTRUCTOR;

end IASW_MAIN;
