
with CSSW_COMMON_TYPES;
with CSSW_IF_TYPES;

package IASW_MRTU is

   package CIT renames CSSW_IF_TYPES;
   package CCT renames CSSW_COMMON_TYPES;

   -- an error occurred while reading analog input:
   EX_ANALOG_INPUT_ERROR : exception;

   -- diagnostic data record:
   type T_MRTU_DIAGNOSTIC is record
       ADC_READ_COUNT,      -- readout counter:
       ADC_ERROR_COUNT,     -- readout error counter:
       ADC_BUSY_COUNT,      -- busy counter:
       ADC_TIMEOUT_COUNT    -- readout timeout counter:
          : CCT.T_UNSIGNED_16;
   end record;


   --------------------------------------------------------------------
   -- Local types and data, made global because of possible patching --
   --------------------------------------------------------------------

   -- timeout reading the analog line:
   ANALOG_INPUT_TIMEOUT : constant CIT.T_RTC_TICKS := 2;

   -- maximum number of analog line acquire retries:
   MRTU_BUSY_RETRIES : constant CCT.T_UNSIGNED_16 := 10;

   -- diagnostic data
   MRTU_DIAGNOSTIC : T_MRTU_DIAGNOSTIC;

   ----------------------------------------------------------------
   -- Local procedures, made global because of possible patching --
   ----------------------------------------------------------------

   procedure CLEAR_DIAGNOSTIC_DATA;

   -----------------------
   -- Global procedures --
   -----------------------

   procedure CONNECT_TO_MRTU;
   function GET_ANALOG_INPUT(CHANNEL_ID : in CIT.T_ANALOG_CHANNEL)
      return CIT.T_ANALOG_VALUE;
   procedure GET_AND_CLEAR_DIAGNOSTIC_DATA(DATA : out T_MRTU_DIAGNOSTIC);
   procedure CONSTRUCTOR;

end IASW_MRTU;
