----------------------------------------------------------------------------
-- Module Name:        IASW_PATCHES - package body
-- Project:            INTEGRAL, JEM-X, DPE
-- Author:             Grzegorz Juchnikowski
-- Creation Date:      16.III.1998
-- Change History:
--    (Version/Date/Author/Description)
-- Function:
--    (Say what the component does)
--    Provides access to the storage of RAM patches loaded to DFEE.
-- Interfaces:
--    (Define the inputs and outputs)
-- Dependencies:
--    (Describe the preconditions for using this component)
--    This package is linked in address state differente than the rest
--    of IASW.  Therefore arguments HDR and DATA should be allocated
--    on the caller side on stack.
-- Processing:
--    (Summarise the processing using pseudo-code or PDL)
--    The storage of patches is a simple array of 150 records containing
--    header (address+length) and data part.  Procedure ADD_PATCH appends
--    a new record to the array.  Procedure GET_PATCH returns given
--    record.
--    Both procedures can face a situation when normally an exception
--    would be raised - ADD_PATCH when storage full or GET_PATCH when index
--    out of range.  Exceptions however can not propagate through differente
--    address states, thus the variable RETCODE is used to return error
--    condition to the caller.
----------------------------------------------------------------------------

with IASW_COMMON;
with IASW_DFEE;

package body IASW_PATCHES is

   procedure CLEAR is
   begin
      NUMBER_DFEE_PATCHES := 0;
   end CLEAR;

   procedure ADD_PATCH(
      HDR  : in IASW_DFEE.T_MEM_BLOCK_SPEC;
      DATA : in IASW_DFEE.T_MEM_BLOCK;
      RETCODE : out T_PATCH_RETCODE
   ) is
   begin
      if NUMBER_DFEE_PATCHES >= K_MAX_DFEE_PATCHES then
         RETCODE := PATCHES_TABLE_FULL;
         return;
         --raise EX_PATCHES_TABLE_FULL;
      end if;
      if (HDR.LENGTH not in 1..IASW_DFEE.K_MAX_LOAD_MEM_BLK_LEN) or
         (DATA'length < HDR.LENGTH)
      then
         RETCODE := PATCHES_LENGTH_ERROR;
         return;
         --raise EX_PATCHES_LENGTH_ERROR;
      end if;
      NUMBER_DFEE_PATCHES := NUMBER_DFEE_PATCHES + 1;
      DFEE_PATCHES(NUMBER_DFEE_PATCHES).HDR := HDR;
      for i in 1..HDR.LENGTH loop
         DFEE_PATCHES(NUMBER_DFEE_PATCHES).MEM(i) := DATA(i);
      end loop;
      RETCODE := PATCHES_OK;
   end ADD_PATCH;

   procedure GET_PATCH(
      NUM  : in INTEGER;
      HDR  : out IASW_DFEE.T_MEM_BLOCK_SPEC;
      DATA : out IASW_DFEE.T_MEM_BLOCK;
      RETCODE : out T_PATCH_RETCODE
   ) is
   begin
      if NUM not in 1..NUMBER_DFEE_PATCHES then
         RETCODE := PATCHES_INDEX_ERROR;
         return;
         --raise EX_PATCHES_INDEX_ERROR;
      end if;
      if (DATA'length < DFEE_PATCHES(NUM).HDR.LENGTH) then
         RETCODE := PATCHES_LENGTH_ERROR;
         return;
         --raise EX_PATCHES_LENGTH_ERROR;
      end if;
      HDR := DFEE_PATCHES(NUM).HDR;
      for i in 1..DFEE_PATCHES(NUM).HDR.LENGTH loop
         DATA(i) := DFEE_PATCHES(NUM).MEM(i);
      end loop;
      RETCODE := PATCHES_OK;
   end GET_PATCH;

   function GET_NUM_PATCHES return INTEGER is
   begin
      return NUMBER_DFEE_PATCHES;
   end GET_NUM_PATCHES;

   procedure CONSTRUCTOR is
   begin
      CLEAR;
   end CONSTRUCTOR;

end IASW_PATCHES;
