
with CSSW_COMMON_TYPES;
with CSSW_IF_TYPES;
with IASW_COMMON;

package IASW_REQ_OUT is

   package CIT renames CSSW_IF_TYPES;
   package CCT renames CSSW_COMMON_TYPES;

   -- exceptions:
   EX_REQUEST_QUEUE_FULL: exception;   -- the request buffer is full
   EX_REQUEST_QUEUE_EMPTY: exception;  -- the request buffer is empty

   -- the codes of request that can be put into the request buffer:
   type T_ON_RQ_TM_REQUEST_CODE is (
       TASK_PARAMETER_REPORT,
       MEMORY_DUMP,
       MEMORY_CRC,
       DIAG_CODE_REPORT
   );

   -- the record identifying a request, a request code is the discriminant,
   -- i.e. it determines which dependent components must appear in the record
   -- value:
   type T_ON_RQ_TM_REQUEST is record
       CODE: T_ON_RQ_TM_REQUEST_CODE;
       OEMF2, OEMF3 : INTEGER;   -- to be reported in OEM in case of failure

       -- TASK_PARAMETER_REPORT =>
       PARAM_ID: IASW_COMMON.T_PARAM_ID;

       -- MEMORY_DUMP | MEMORY_CRC =>
       MEM_HDR: IASW_COMMON.T_TC_MEM_HDR;

       -- DIAG_CODE_REPORT =>
       DIAG_CODE : CCT.T_UNSIGNED_16;
       DIAG_ID   : CCT.T_UNSIGNED_16;

   end record;

   --------------------------------------------------------------------
   -- Local types and data, made global because of possible patching --
   --------------------------------------------------------------------

   -- the range of the request buffer size:
   subtype T_BUFFER_SIZE is INTEGER range 0..10;

   -- request buffer, FIFO type:
   REQUEST_QUEUE : array (0..T_BUFFER_SIZE'last-1) of T_ON_RQ_TM_REQUEST;

   -- position of the first valid request in the request buffer:
   FIRST_REQUEST : T_BUFFER_SIZE;

   -- quantity of valid requests stored in the request buffer,
   -- when REQUEST_COUNT is equal to 0 the FIRST_REQUEST is undefined:
   REQUEST_COUNT : T_BUFFER_SIZE;


   ----------------------------------------------------------------
   -- Local procedures, made global because of possible patching --
   ----------------------------------------------------------------

   procedure CLEAR_REQUEST_BUFFER;
   procedure GET_REQUEST(
      REQUEST : out T_ON_RQ_TM_REQUEST;
      PARTIAL : out BOOLEAN );
   procedure REPORT_TASK_PARAMS(REQUEST : in T_ON_RQ_TM_REQUEST);
   procedure DUMP_MEMORY(REQUEST: in T_ON_RQ_TM_REQUEST);
   procedure CALCULATE_MEMORY_CRC(REQUEST: in T_ON_RQ_TM_REQUEST);
   procedure DO_DIAG_CODE_REPORT(REQUEST: in T_ON_RQ_TM_REQUEST);

   -----------------------
   -- Global procedures --
   -----------------------

   procedure ADD_REQUEST(REQUEST: in T_ON_RQ_TM_REQUEST);

   procedure TASK_REQ_OUT;

   function REQUESTS_QUEUE_FULL return BOOLEAN;

   procedure REPORT_DIAG_CODE(
      DIAG_CODE : in CCT.T_UNSIGNED_16;
      DIAG_ID   : in CCT.T_UNSIGNED_16 := 16#FFFF# );

   procedure CONSTRUCTOR;

end IASW_REQ_OUT;
