
with SYSTEM;
with CSSW_COMMON_TYPES;
with CSSW_IF_TYPES;
with IASW_COMMON;
with IASW_DFEE;
with IASW_MODES;

package IASW_STATE is

package CCT renames CSSW_COMMON_TYPES;
package CIT renames CSSW_IF_TYPES;

   type T_DIAGNOSTIC_HK is record
      BCPKT_FILTER,
      BCPKT_CONDITION,
      HK_FILTER,
      HK_CONDITION
        : CCT.T_UNSIGNED_16;
   end record;

   --------------------------------------------------------------------
   -- Local types and data, made global because of possible patching --
   --------------------------------------------------------------------

   -- values used in the field MODE in the TC(5,5).
   -- they are very similar to constants IASW_DFEE.K_STATE_xxxx, but
   -- are defined separatelly because of modes for test variants.
   -- Test modes are not provided in IASW_DEFF.
   K_MODE_SAFE:             constant CCT.T_UNSIGNED_8 := 1;
   K_MODE_MEMORY:           constant CCT.T_UNSIGNED_8 := 2;
   K_MODE_SETUP:            constant CCT.T_UNSIGNED_8 := 5;
   K_MODE_DATA_TAKING:      constant CCT.T_UNSIGNED_8 := 10;
   K_MODE_CALIBRATION:      constant CCT.T_UNSIGNED_8 := 20;
   K_MODE_DIAGNOSTIC:       constant CCT.T_UNSIGNED_8 := 40;
   K_MODE_DATA_TAKING_TEST: constant CCT.T_UNSIGNED_8 := 11;
   K_MODE_CALIBRATION_TEST: constant CCT.T_UNSIGNED_8 := 21;
   K_MODE_DIAGNOSTIC_TEST:  constant CCT.T_UNSIGNED_8 := 41;

   -- can be changed with use of TC(5,3)/TID=1/FID=14
   DET_PRESS_1_MIN :  CCT.T_UNSIGNED_16 := 0;
   DET_PRESS_2_MIN :  CCT.T_UNSIGNED_16 := 0;

   -- can be changed with use of TC(5,3)/TID=1/FID=24
   DET_TEMP_1_MAX :  CCT.T_UNSIGNED_16 := 255;
   DET_TEMP_2_MAX :  CCT.T_UNSIGNED_16 := 255;

   K_DIG_P5_V_MIN:  constant CCT.T_UNSIGNED_16 := 160;
   K_DIG_P5_V_MAX:  constant CCT.T_UNSIGNED_16 := 255;
   K_DIG_P5_A_MIN:  constant CCT.T_UNSIGNED_16 := 0;
   K_DIG_P5_A_MAX:  constant CCT.T_UNSIGNED_16 := 255;

   K_ANA_P5_V_MIN:  constant CCT.T_UNSIGNED_16 := 160;  -- Volt.+5 analog
   K_ANA_P5_V_MAX:  constant CCT.T_UNSIGNED_16 := 255;

   K_ANA_M5_V_MIN:  constant CCT.T_UNSIGNED_16 := 160;  -- Volt.-5 analog
   K_ANA_M5_V_MAX:  constant CCT.T_UNSIGNED_16 := 255;

   K_P12V_V_MIN: constant CCT.T_UNSIGNED_16 := 160;
   K_P12V_V_MAX: constant CCT.T_UNSIGNED_16 := 255;
   K_P12V_A_MIN: constant CCT.T_UNSIGNED_16 := 0;
   K_P12V_A_MAX: constant CCT.T_UNSIGNED_16 := 255;

   K_M12V_V_MIN: constant CCT.T_UNSIGNED_16 := 160;
   K_M12V_V_MAX: constant CCT.T_UNSIGNED_16 := 255;
   K_M12V_A_MIN: constant CCT.T_UNSIGNED_16 := 0;
   K_M12V_A_MAX: constant CCT.T_UNSIGNED_16 := 255;

   -- There must be at least RADMON_ENTRY_CNT consecutive occurencies of
   -- high value of the radiation counter to consider high radiation.
   -- There must be at least RADMON_EXIT_CNT consecutive occurencies of
   -- low value of the radiation counter to consider low radiation.
   -- Both values are commandable with TC(5,3)/TID=1/FID=13.
   RADMON_ENTRY_CNT : CCT.T_UNSIGNED_16 := 2;
   RADMON_EXIT_CNT  : CCT.T_UNSIGNED_16 := 75;

   -- the undefined/invalid time:
   K_OBT_UNDEFINED       : constant CIT.T_OBT := (0,0,0);
   K_BCPKT_OBT_UNDEFINED : constant IASW_COMMON.T_BCPKT_OBT := (0,0);

   subtype T_SHUTDOWN_LEVEL is CCT.T_UNSIGNED_16 range 0..16#80#;

   type T_TIME_INTERVAL is record
      ENTRY_TIME, EXIT_TIME : CIT.T_OBT;
      ENTRY_OK, EXIT_OK : BOOLEAN;
      --methodes associated with this object:
      --procedure INTERVAL_INIT( INTERVAL : out T_TIME_INTERVAL );
      --procedure INTERVAL_SET( INTERVAL : in out T_TIME_INTERVAL;
      --   T1,T2 : in IASW_COMMON.T_BCPKT_OBT );
      --function TIME_INSIDE_INTERVAL( INTERVAL : in T_TIME_INTERVAL;
      --   TIME : in CIT.T_OBT ) return BOOLEAN;
      --private:
      --function BCPKT_TO_RBI_OBT(OBT1 : in IASW_COMMON.T_BCPKT_OBT) return CIT.T_OBT;
   end record;

   subtype T_RAD_MONITOR_COUNTER is CCT.T_UNSIGNED_16;

   type T_RAD_MONITOR is record
      VALUE, HIGH_LIMIT, LOW_LIMIT : T_RAD_MONITOR_COUNTER;
      IS_HIGH : BOOLEAN;
      CNT_OPPOSIT : CCT.T_UNSIGNED_16;
      --methodes associated with this object:
      --procedure RADMON_INIT(RM : in out T_RAD_MONITOR;
      --  HL,LL : in T_RAD_MONITOR_COUNTER);
      --procedure RADMON_NEW_VALUE(RM : in out T_RAD_MONITOR;
      --  V : in T_RAD_MONITOR_COUNTER);
      --procedure RADMON_BCP1_TICK( RM : in T_RAD_MONITOR );
      --function RADMON_HIGH(RM : in T_RAD_MONITOR;
      --  FILTER, CONDITION : in BOOLEAN) return BOOLEAN;
   end record;

   -- note: for every flag in the record below TRUE means the error condition
   type T_BCPKT_FLAGS is record
      OTF,
      AOCS_MODE,
      AOCS_SUBMODE,
      ESAM,
      RAD_MONITOR_1,
      RAD_MONITOR_2,
      RAD_MONITOR_3,
      RAD_BELTS,
      ECLIPSE,
      TM_SHARE,
      SWITCH_OFF_IMM,
      SPARE11, SPARE12, SPARE13, SPARE14, SPARE15
         : BOOLEAN;
   end record;
   for T_BCPKT_FLAGS use record
      OTF             at 0 range 0..0;
      AOCS_MODE       at 0 range 1..1;
      AOCS_SUBMODE    at 0 range 2..2;
      ESAM            at 0 range 3..3;
      RAD_MONITOR_1   at 0 range 4..4;
      RAD_MONITOR_2   at 0 range 5..5;
      RAD_MONITOR_3   at 0 range 6..6;
      RAD_BELTS       at 0 range 7..7;
      ECLIPSE         at 0 range 8..8;
      TM_SHARE        at 0 range 9..9;
      SWITCH_OFF_IMM  at 0 range 10..10;
      SPARE11         at 0 range 11..11;
      SPARE12         at 0 range 12..12;
      SPARE13         at 0 range 13..13;
      SPARE14         at 0 range 14..14;
      SPARE15         at 0 range 15..15;
   end record;

   type T_HK_FLAGS is record
      DET_PRESSURE_1,
      DET_PRESSURE_2,
      DET_TEMP_1,
      DET_TEMP_2,
      VOLTAGE_P5V_DIG,
      CURRENT_P5V_DIG,
      VOLTAGE_P5V_ANA,
      VOLTAGE_M5V_ANA,
      VOLTAGE_P12V,
      CURRENT_P12V,
      VOLTAGE_M12V,
      CURRENT_M12V,
      SPARE12, SPARE13, SPARE14, SPARE15
         : BOOLEAN;
   end record;
   for T_HK_FLAGS use record
      DET_PRESSURE_1  at 0 range 0..0;
      DET_PRESSURE_2  at 0 range 1..1;
      DET_TEMP_1      at 0 range 2..2;
      DET_TEMP_2      at 0 range 3..3;
      VOLTAGE_P5V_DIG at 0 range 4..4;
      CURRENT_P5V_DIG at 0 range 5..5;
      VOLTAGE_P5V_ANA at 0 range 6..6;
      VOLTAGE_M5V_ANA at 0 range 7..7;
      VOLTAGE_P12V    at 0 range 8..8;
      CURRENT_P12V    at 0 range 9..9;
      VOLTAGE_M12V    at 0 range 10..10;
      CURRENT_M12V    at 0 range 11..11;
      SPARE12         at 0 range 12..12;
      SPARE13         at 0 range 13..13;
      SPARE14         at 0 range 14..14;
      SPARE15         at 0 range 15..15;
   end record;

   -- HV settings (set before shutdown)
   type T_DFEE_HV is record
      SETTINGS : IASW_DFEE.T_HW_SETTINGS;
   end record;

   -- structure of data part of TC(5,5)
   type T_TC55_DATA is record
      MODE    : CCT.T_UNSIGNED_8;
      PARAM_1 : CCT.T_UNSIGNED_8;
      PARAM_2 : CCT.T_UNSIGNED_16;
      PARAM_3 : CCT.T_UNSIGNED_16;
      PARAM_4 : CCT.T_UNSIGNED_16;
      PARAM_5 : CCT.T_UNSIGNED_16;
   end record;
   for T_TC55_DATA use record
      MODE    at 0 range 0..7;
      PARAM_1 at 0 range 8..15;
      PARAM_2 at 1 range 0..15;
      PARAM_3 at 2 range 0..15;
      PARAM_4 at 3 range 0..15;
      PARAM_5 at 4 range 0..15;
   end record;

   type T_AUTO_RECOVERY_CONTROL_INFO is record
      RECOVERY_LEVEL : CCT.T_UNSIGNED_8;
      CPU_MODE : CCT.T_UNSIGNED_8;
   end record;
   for T_AUTO_RECOVERY_CONTROL_INFO use record
      RECOVERY_LEVEL at 0 range 0..7;
      CPU_MODE       at 0 range 8..15;
   end record;

   -- values of RECOVERY_LEVEL
   K_RECOVER_NONE        : constant := 0;
   K_RECOVER_CONTEXT     : constant := 3;
   K_RECOVER_HV          : constant := 6;
   K_RECOVER_DATA_TAKING : constant := 9;

   type T_RECOVERY_OEM_CONTROL_INFO is record
      OEM_SENT : BOOLEAN;
      ASL : T_SHUTDOWN_LEVEL;
      WSL : T_SHUTDOWN_LEVEL;
   end record;


   -- variables:

   AUTO_RECOVERY_CONTROL_INFO : T_AUTO_RECOVERY_CONTROL_INFO;  -- set from ground
   REC_OEM_CONTROL : T_RECOVERY_OEM_CONTROL_INFO;  -- regulates generation of OEM-s(recovery)

   -- operator's overrides:
   BCPKT_FILTER, BCPKT_CONDITION : T_BCPKT_FLAGS;
   HK_FILTER, HK_CONDITION : T_HK_FLAGS;

   -- data related to the DPE state:
   WANTED_SHUTDOWN_LEVEL,    -- a function of external conditions
   ACTUAL_SHUTDOWN_LEVEL,    -- current state in process of approaching to the wanted level
   MINIMAL_SHUTDOWN_LEVEL    -- minimal allowed because of malfunction
      : T_SHUTDOWN_LEVEL;
   LOW_VOLTAGE_FAILURE, PRESSURE_FAILURE, TEMP_FAILURE : BOOLEAN;

   -- last valid Broadcast Packet:
   BCPKT : IASW_COMMON.T_BROADCAST_PACKET;
   BCPKT_VALID : BOOLEAN;
   ECLIPSE_INTERVAL, RADIATION_BELTS_INTERVAL : T_TIME_INTERVAL;
   RADMON : array(1..3) of T_RAD_MONITOR;

   -- data for detecting of a new pointing
   PREV_PID, PREV_VALID_PID : IASW_COMMON.T_PID;
   PREV_OTF : BOOLEAN;

   -- parameters of the interrupted state
   INTERRUPTED_DFEE_STATE : IASW_DFEE.T_DFEE_STATE;
   SCF1,SCF2 : IASW_MODES.T_SCIENCE_FORMAT;
   GFL1,GFL2 : CCT.T_UNSIGNED_16;
   DFEE_HV : T_DFEE_HV;
   NEED_HV_ON : BOOLEAN;

   --DFEE_LIVED_AT_LEAST_ONCE : BOOLEAN;  -- set after recpt. of HK, never reset
   --in ver.1.63 moved to IASW_DFEE

   PENDING_ANALYSE : BOOLEAN;
   STATE_SEMAPHORE : CCT.T_UNSIGNED_16 := 0;

   --LV_FAILURE_CNT : INTEGER;
   --deleted in ver.1.62 (this variable not used)

   ----------------------------------------------------------------
   -- Local procedures, made global because of possible patching --
   ----------------------------------------------------------------

   function CAN_ENTER_STATE_CRITSEC return BOOLEAN;
   function DEEPER_SHUTDOWN_LEVEL return T_SHUTDOWN_LEVEL;
   procedure RADMON_INIT(
      RM : out T_RAD_MONITOR;
      HL,LL : in T_RAD_MONITOR_COUNTER );
   procedure RADMON_NEW_VALUE(
      RM : in out T_RAD_MONITOR;
      V : in T_RAD_MONITOR_COUNTER );
   procedure RADMON_BCP1_TICK( RM : in out T_RAD_MONITOR );
   function RADMON_HIGH(
      RM : in T_RAD_MONITOR;
      FILTER, CONDITION : in BOOLEAN
   ) return BOOLEAN;
   function BCPKT_TO_RBI_OBT(
      OBT1 : in IASW_COMMON.T_BCPKT_OBT
   ) return CIT.T_OBT;
   procedure INTERVAL_INIT(
      INTERVAL : out T_TIME_INTERVAL );
   procedure INTERVAL_SET(
      INTERVAL : in out T_TIME_INTERVAL;
      T1,T2 : in IASW_COMMON.T_BCPKT_OBT );
   function TIME_INSIDE_INTERVAL(
      INTERVAL : in T_TIME_INTERVAL;
      TIME : in CIT.T_OBT
   ) return BOOLEAN;
   procedure CHANGE_STATE(
      TC_LEN:    in INTEGER;
      MODE:      in CCT.T_UNSIGNED_8;
      PARAM_1:   in CCT.T_UNSIGNED_8  := 0;
      PARAM_2:   in CCT.T_UNSIGNED_16 := 0;
      PARAM_3:   in CCT.T_UNSIGNED_16 := 0;
      PARAM_4:   in CCT.T_UNSIGNED_16 := 0;
      PARAM_5:   in CCT.T_UNSIGNED_16 := 0;
      OEM_RA6:   in BOOLEAN := TRUE );
   procedure SHUTDOWN(
      ACTUAL_LEVEL: in out T_SHUTDOWN_LEVEL;
      WANTED_LEVEL: in T_SHUTDOWN_LEVEL );
   procedure RESTORE(
      ACTUAL_LEVEL : in out T_SHUTDOWN_LEVEL;
      WANTED_LEVEL : in T_SHUTDOWN_LEVEL );
   function START_OF_A_NEW_POINTING return BOOLEAN;
   procedure RESTART_DATA_TAKING;
   function SWITCH_OFF_IMM return BOOLEAN;
   function ESAM return BOOLEAN;
   function ECLIPSE return BOOLEAN;
   function RADIATION_BELTS return BOOLEAN;
   function TM_SHARE return CCT.T_UNSIGNED_16;
   procedure ANALYSE_STATE;
   function MAX_ALLOWED_STATE( LEVEL : in T_SHUTDOWN_LEVEL ) return CCT.T_UNSIGNED_8;
   procedure ANALYSE_LOW_VOLTAGE( MRTU_HK : in IASW_COMMON.T_MRTU_HK );
   procedure ANALYSE_PRESSURE   ( MRTU_HK : in IASW_COMMON.T_MRTU_HK );
   procedure ANALYSE_TEMPERATURE( MRTU_HK : in IASW_COMMON.T_MRTU_HK );


   -----------------------
   -- Global procedures --
   -----------------------

   procedure ENTER_STATE_CRITSEC;
   procedure LEAVE_STATE_CRITSEC;

   procedure TASK_STATE;

   procedure TC55(TC_DATA: in CIT.T_TC_APPL_DATA_FIELD);

   procedure SET_WANTED_SHUTDOWN_LEVEL;

   function HAVE_PENDING_ANALYSE return BOOLEAN;

   procedure ANALYSE_MRTU_DATA( MRTU_HK : in IASW_COMMON.T_MRTU_HK );

   procedure UPDATE_BCPKT(NEW_BCPKT: in IASW_COMMON.T_BROADCAST_PACKET);

   procedure GET_LAST_BCPKT(LAST_BCPKT: out IASW_COMMON.T_BROADCAST_PACKET; VALID: out BOOLEAN);

   procedure GET_DPE_STATE(DS: out IASW_COMMON.T_DPE_STATE);

   procedure CHECK_TC_CORRECTNESS_VERSUS_STATE(TC : in CIT.T_TCSP);

   procedure BCP1_TICK;

   procedure SET_SW_PARAM(
      PARAM_ID : in IASW_COMMON.T_FID;
      PARAM_VAL: in IASW_COMMON.T_TC_PARAM_VAL;
      PARAM_LEN: in IASW_COMMON.T_SW_PARAM_LEN);

   procedure GET_SW_PARAM(
      PARAM_ID : in  IASW_COMMON.T_FID;
      PARAM_VAL: out IASW_COMMON.T_TM_PARAM_VAL;
      PARAM_LEN: out IASW_COMMON.T_SW_PARAM_LEN);

   procedure GET_HK_DIAG(
      FLAGS : out IASW_COMMON.T_16_BITS;
      DIAG  : out T_DIAGNOSTIC_HK);

   procedure CONSTRUCTOR;

end IASW_STATE;
