with UNCHECKED_CONVERSION;
with CSSW_COMMON_TYPES;
with CSSW_IF_TYPES;
with IASW_COMMON;

package IASW_TM is

   package CIT renames CSSW_IF_TYPES;
   package CCT renames CSSW_COMMON_TYPES;

   type T_OEM_INDICATOR is (
      -- telecommand rejected because of:
      EV_TC_REJ_TYPE_SUBTYPE,
      EV_TC_REJ_TID_FID,
      EV_TC_REJ_MID,
      EV_TC_REJ_ADDRESS,     -- error of the field ADDRESS
      EV_TC_REJ_LENGTH,      -- error of the fild LENGTH or length of TC itself
      EV_TC_REJ_DATA,        -- some other error in the command text
      EV_TC_REJ_BUSY,        -- queue of commands full
      EV_TC_REJ_SAFETY,      -- command illegal because of shutdown action
      EV_TC_REJ_STATE,       -- state changing illegal because of shutdown action
      EV_TC_REJ_EXEC_FAILED, -- execution of command started, but failed
      -- malfunctions of the hardware
      EV_HSL_OVERFLOW,
      EV_HSL_BUSY,
      EV_HSL_CRC,
      EV_HSL_TIMEOUT,
      EV_HSL_LOST_SYNCHRO,
      EV_LSL_OVERRUN,
      EV_LSL_FRAMING,
      EV_LSL_PARITY,
      EV_LSL_DISABLED,
      EV_LSL_BUSY,
      EV_LSL_TIMEOUT,
      EV_MRTU_ERROR,
      EV_MRTU_BUSY,
      EV_MRTU_TIMEOUT,
      -- problems with communication with DFEE
      EV_DFEE_UNEXP_ANSWER,
      EV_DFEE_RA_1,
      EV_DFEE_RA_2,
      EV_DFEE_RA_3,
      EV_DFEE_RA_4,
      EV_DFEE_RA_5,
      EV_DFEE_RA_6,
      EV_DFEE_RA_X,
      EV_DFEE_CRC,
      EV_DFEE_STATE,
      EV_DFEE_HK_CC,
      EV_DFEE_CONTEXT,
      EV_DFEE_HV_OFF_RELAY,
      -- problems with BCP1
      EV_CLOCK_BCP1_SHORT,   -- BCP1 shorter than 7.5 sec
      EV_CLOCK_BCP1_LONG,    -- no BCP1 after 8.5 sec timeout
      -- unexpected behaviour of IASW (should never happen)
      EV_IASW_EXCEPTION,
      EV_IASW_BUFFER,
      EV_IASW_REQ_QUEUE,
      EV_IASW_PATCHES,
      -- unexpected behaviour of CSSW (should never happen)
      EV_CSSW_EXCEPTION,
      EV_CSSW_NO_RTC,        -- no RTC event (hardly possible)
      -- events while DPE does his autonomous actions
      EV_AUTO_SHUTDOWN,      -- shutdown initiated
      EV_AUTO_SHD_FAILED,    -- shutdown failed
      EV_AUTO_RECOVERY,      -- recovery initiated
      EV_AUTO_RCV_FAILED,    -- recovery failed
      EV_AUTO_RESTART,       -- restart data taking because of new pointing
      EV_TM_LOST,            --
      EV_ERROR_CONDITION_OFF
   );

   -- structure of parameter DIAG in procedure GET_HK_DIAG().
   type T_DIAGNOSTIC_HK is record
      OEMS_REJECTED : CCT.T_UNSIGNED_16;
   end record;

   --------------------------------------------------------------------
   -- Local types and data, made global because of possible patching --
   --------------------------------------------------------------------

   -- flags used for filtering of OEMs - one flag for each OEM class.
   -- The flag is set at emitting of OEM, and cleared at BCP1.
   -- At PUT_OEM(), if the flag is set, disables emiting the OEM.
   ERROR_FLAG : array (T_OEM_INDICATOR) of BOOLEAN;

   type T_OEM_PARAMS is record
      FIELD2 : INTEGER;
      FIELD3 : INTEGER;
   end record;

   -- used in filtering of OEM EV_DFEE_STATE.
   LAST_OEM_DFEE_STATE : T_OEM_PARAMS;

   -- counter of OEMs filtered out
   OEMS_REJECTED : CCT.T_UNSIGNED_16;

   type T_OEM_DESCRIPTOR is record
      ID : CIT.T_IASW_ON_EVENT_ID;
      F1 : CCT.T_UNSIGNED_6;
      CL : CIT.T_ON_EVENT_MSG_CLASS;
   end record;

   OEM_DESCRIPTOR: constant array (T_OEM_INDICATOR) of T_OEM_DESCRIPTOR := (
      EV_TC_REJ_TYPE_SUBTYPE => (ID => 128, F1 => 1,  CL => CIT.FAILED_TC_REPORT),
      EV_TC_REJ_TID_FID      => (ID => 129, F1 => 2,  CL => CIT.FAILED_TC_REPORT),
      EV_TC_REJ_MID          => (ID => 130, F1 => 3,  CL => CIT.FAILED_TC_REPORT),
      EV_TC_REJ_ADDRESS      => (ID => 131, F1 => 4,  CL => CIT.FAILED_TC_REPORT),
      EV_TC_REJ_LENGTH       => (ID => 132, F1 => 5,  CL => CIT.FAILED_TC_REPORT),
      EV_TC_REJ_DATA         => (ID => 133, F1 => 6,  CL => CIT.FAILED_TC_REPORT),
      EV_TC_REJ_BUSY         => (ID => 134, F1 => 7,  CL => CIT.FAILED_TC_REPORT),
      EV_TC_REJ_SAFETY       => (ID => 135, F1 => 8,  CL => CIT.FAILED_TC_REPORT),
      EV_TC_REJ_STATE        => (ID => 136, F1 => 9,  CL => CIT.FAILED_TC_REPORT),
      EV_TC_REJ_EXEC_FAILED  => (ID => 137, F1 => 10, CL => CIT.FAILED_TC_REPORT),
      EV_HSL_OVERFLOW        => (ID => 151, F1 => 1,  CL => CIT.MAJOR_ANOMALY),
      EV_HSL_BUSY            => (ID => 157, F1 => 7,  CL => CIT.EXCEPTN),
      EV_HSL_CRC             => (ID => 158, F1 => 8,  CL => CIT.MAJOR_ANOMALY),
      EV_HSL_TIMEOUT         => (ID => 159, F1 => 9,  CL => CIT.MAJOR_ANOMALY),
      EV_HSL_LOST_SYNCHRO    => (ID => 154, F1 => 11, CL => CIT.EVENTS),
      EV_LSL_OVERRUN         => (ID => 161, F1 => 1,  CL => CIT.MAJOR_ANOMALY),
      EV_LSL_FRAMING         => (ID => 162, F1 => 2,  CL => CIT.EVENTS),
      EV_LSL_PARITY          => (ID => 163, F1 => 3,  CL => CIT.EVENTS),
      EV_LSL_DISABLED        => (ID => 166, F1 => 6,  CL => CIT.EXCEPTN),
      EV_LSL_BUSY            => (ID => 167, F1 => 7,  CL => CIT.EXCEPTN),
      EV_LSL_TIMEOUT         => (ID => 169, F1 => 9,  CL => CIT.EVENTS),
      EV_MRTU_ERROR          => (ID => 175, F1 => 5,  CL => CIT.MAJOR_ANOMALY),
      EV_MRTU_BUSY           => (ID => 177, F1 => 7,  CL => CIT.MAJOR_ANOMALY),
      EV_MRTU_TIMEOUT        => (ID => 179, F1 => 9,  CL => CIT.MAJOR_ANOMALY),
      EV_DFEE_UNEXP_ANSWER   => (ID => 180, F1 => 0,  CL => CIT.MAJOR_ANOMALY),
      EV_DFEE_RA_1           => (ID => 181, F1 => 1,  CL => CIT.EVENTS),
      EV_DFEE_RA_2           => (ID => 182, F1 => 2,  CL => CIT.EVENTS),
      EV_DFEE_RA_3           => (ID => 183, F1 => 3,  CL => CIT.EVENTS),
      EV_DFEE_RA_4           => (ID => 184, F1 => 4,  CL => CIT.EVENTS),
      EV_DFEE_RA_5           => (ID => 185, F1 => 5,  CL => CIT.EVENTS),
      EV_DFEE_RA_6           => (ID => 186, F1 => 6,  CL => CIT.EVENTS),
      EV_DFEE_RA_X           => (ID => 187, F1 => 7,  CL => CIT.MAJOR_ANOMALY),
      EV_DFEE_CRC            => (ID => 188, F1 => 8,  CL => CIT.EVENTS),
      EV_DFEE_STATE          => (ID => 189, F1 => 9,  CL => CIT.EVENTS),
      EV_DFEE_HK_CC          => (ID => 190, F1 => 10, CL => CIT.EVENTS),
      EV_DFEE_CONTEXT        => (ID => 191, F1 => 11, CL => CIT.EVENTS),
      EV_DFEE_HV_OFF_RELAY   => (ID => 192, F1 => 12, CL => CIT.MAJOR_ANOMALY),
      EV_CLOCK_BCP1_SHORT    => (ID => 201, F1 => 1,  CL => CIT.MAJOR_ANOMALY),
      EV_CLOCK_BCP1_LONG     => (ID => 202, F1 => 2,  CL => CIT.MAJOR_ANOMALY),
      EV_IASW_EXCEPTION      => (ID => 211, F1 => 1,  CL => CIT.EXCEPTN),
      EV_IASW_BUFFER         => (ID => 212, F1 => 2,  CL => CIT.MAJOR_ANOMALY),
      EV_IASW_REQ_QUEUE      => (ID => 213, F1 => 3,  CL => CIT.MAJOR_ANOMALY),
      EV_IASW_PATCHES        => (ID => 214, F1 => 4,  CL => CIT.MAJOR_ANOMALY),
      EV_CSSW_EXCEPTION      => (ID => 221, F1 => 1,  CL => CIT.EXCEPTN),
      EV_CSSW_NO_RTC         => (ID => 222, F1 => 2,  CL => CIT.MAJOR_ANOMALY),
      EV_AUTO_SHUTDOWN       => (ID => 231, F1 => 1,  CL => CIT.EVENTS),
      EV_AUTO_SHD_FAILED     => (ID => 232, F1 => 2,  CL => CIT.EVENTS),
      EV_AUTO_RECOVERY       => (ID => 233, F1 => 3,  CL => CIT.EVENTS),
      EV_AUTO_RCV_FAILED     => (ID => 234, F1 => 4,  CL => CIT.EVENTS),
      EV_AUTO_RESTART        => (ID => 235, F1 => 5,  CL => CIT.EVENTS),
      EV_TM_LOST             => (ID => 241, F1 => 1,  CL => CIT.EVENTS),
      EV_ERROR_CONDITION_OFF => (ID => 254, F1 => 0,  CL => CIT.EVENTS)
   );

   -- TM sequence counters for each of the APIDs separately.
   subtype T_TM_APID_RANGE is
      CIT.T_IASW_APID range CIT.K_IASW_LOW_APID..CIT.K_IASW_HIGH_APID;
   SEQ_COUNT : array(T_TM_APID_RANGE) of CIT.T_SOURCE_SEQ_COUNT;

   ----------------------------------------------------------------
   -- Local procedures, made global because of possible patching --
   ----------------------------------------------------------------

   function CVT is new UNCHECKED_CONVERSION(
      SOURCE => CIT.T_IASW_ON_EVENT_ID, TARGET => INTEGER);

   procedure CLEAR_OEMS_FILTER;

   function MASTER_ERROR_FLAG return BOOLEAN;

   function OEM_FILTERED_OUT(
      INDIC  : in T_OEM_INDICATOR;
      FIELD2 : in INTEGER;
      FIELD3 : in INTEGER
   ) return BOOLEAN;

   -----------------------
   -- Global procedures --
   -----------------------

   procedure INIT_SCIENCE_TM(
      DATA : out CIT.T_SCIENCE_DATA_FIELD);

   procedure INIT_ON_REQUEST_TM(
      DATA : out CIT.T_SCIENCE_DATA_FIELD;
      TYP  : in IASW_COMMON.T_PACKET_TYPE;
      SUB  : in IASW_COMMON.T_PACKET_SUBTYPE);

   procedure PUT_TM(
      APID : in CIT.T_IASW_APID;
      DATA : in CIT.T_SCIENCE_DATA_FIELD;
      DONE : out BOOLEAN;
      SCIENCE: in BOOLEAN := TRUE);

   procedure PUT_TM_FOR_SURE(
      APID : in CIT.T_IASW_APID;
      DATA : in CIT.T_SCIENCE_DATA_FIELD;
      SCIENCE: in BOOLEAN := TRUE);

   procedure PUT_OEM(
      INDIC  : in T_OEM_INDICATOR;
      FIELD2 : in INTEGER := 0;
      FIELD3 : in INTEGER := 0 );

   procedure LSL_ERROR_CONDITION_OFF;
   procedure DFEE_CRC_ERROR_CONDITION_OFF;
   procedure DFEE_STATE_ERROR_CONDITION_OFF;
   procedure HSL_ERROR_CONDITION_OFF;
   procedure MRTU_ERROR_CONDITION_OFF;
   procedure BCP1_ERROR_CONDITION_OFF;

   procedure BCP1_TICK;

   procedure GET_HK_DIAG(
      FLAGS : out IASW_COMMON.T_16_BITS;
      DIAG  : out T_DIAGNOSTIC_HK);

   procedure CONSTRUCTOR;

end IASW_TM;
